import { system, world, EquipmentSlot, EntityComponentTypes, ItemComponentTypes, ItemStack } from "@minecraft/server";

world.afterEvents.entityDie.subscribe((event) => {
    const equipableComponent = event.damageSource.damagingEntity.getComponent(EntityComponentTypes.Equippable);

    if (equipableComponent) {
        const mainHandItem = equipableComponent.getEquipment(EquipmentSlot.Mainhand);

        if (mainHandItem.typeId == "hightideba:undead_sword") {
            if (event.deadEntity.getTags().includes("wasSwordRevived")) return; // Check if the entity has already been revived
            if (event.deadEntity.typeId == "minecraft:wither" || event.deadEntity.typeId == "minecraft:ender_dragon") return; // Check if the entity is a boss entity

            const respawnSoundOptions = {
                "volume": 1.0,
                "pitch": 0.6
            };

            let newEntity = world.getDimension(event.deadEntity.dimension.id).spawnEntity(event.deadEntity.typeId, event.deadEntity.location);
            newEntity.addTag("wasSwordRevived");
            world.playSound("mob.wither.shoot", event.deadEntity.location, respawnSoundOptions);
        }
    }
});

world.afterEvents.playerJoin.subscribe((event) => {
    var playerList = world.getAllPlayers();
    var targetPlayerId = -1;

    for (var i = 0; i < playerList.length; i++) {
        if (playerList[i].name == event.playerName) {
            targetPlayerid = i;
            break;
        }
    }

    if (targetPlayerId != -1) {
        if (!playerList[targetPlayerId].hasTag("got_hightide_credits_paper")) {
            playerList[targetPlayerId].triggerEvent("hightideba:give_credits_paper");
        }
    }
});

world.beforeEvents.itemUseOn.subscribe((event) => {
    let shovel_items = [
        "minecraft:wooden_shovel",
        "minecraft:stone_shovel",
        "minecraft:golden_shovel",
        "minecraft:iron_shovel",
        "minecraft:diamond_shovel",
        "minecraft:netherite_shovel"
    ];

    if (event.block.typeId == "minecraft:pumpkin") {
        if (shovel_items.includes(event.itemStack.typeId)) {
            let currentDropMultiplier = 3; // 3 is the default amount of pumpkin filler items that will drop
            const enchantableComponent = event.itemStack.getComponent(ItemComponentTypes.Enchantable);
            if (enchantableComponent) {
                const fortuneEnchantment = enchantableComponent.getEnchantment("fortune");
                if (fortuneEnchantment) {
                    currentDropMultiplier *= fortuneEnchantment.level;
                }
            }

            const pumpkinFillerDrop = new ItemStack("hightideba:pumpkin_filler", currentDropMultiplier);
            system.run(() => {
                world.getDimension(event.block.dimension.id).setBlockType(event.block.location, "minecraft:carved_pumpkin");
                world.getDimension(event.block.dimension.id).spawnItem(pumpkinFillerDrop, event.block.location);
            });
        }
    }
});

world.afterEvents.itemUse.subscribe((event) => {
    if (event.itemStack.typeId == "hightideba:crystal_cube") {
        const speedEffectOptions = {
            "amplifier": 5,
            "showParticles": true
        };

        system.run(() => {
            world.getDimension(event.source.dimension.id).runCommand(`execute positioned ${event.source.location.x} ${event.source.location.y} ${event.source.location.z} run clear @p[hasitem={item=hightideba:crystal_cube}] hightideba:crystal_cube 0 1`);
        });
        event.source.addEffect("invisibility", 600);
        event.source.addEffect("speed", 600, speedEffectOptions);
        event.source.addEffect("night_vision", 600);
        event.source.extinguishFire();
    }
    else if (event.itemStack.typeId == "hightideba:credits_paper") {
        world.sendMessage(`Hello ${event.source.name}!\nI want to thank you for checking out and using my add-on.\n\nThis add-on has taken me an insane amount of time to make and lots of time to troubleshoot. If you enjoy it, please consider donating to my Kofi page.\n\nHave fun playing with this addon!`);
    }
});

/*world.afterEvents.playerBreakBlock.subscribe((event) => {
    const tool_tiers = [
        "minecraft:wooden_",
        "minecraft:stone_",
        "minecraft:iron_",
        "minecraft:golden_",
        "minecraft:diamond_",
        "minecraft:netherite_"
    ];

    var currently_held_tool = "NONE";

    if (event.block.typeId == "hightideba:blue_obsidian") {
        const tierRequiredForBreak = 4; // The level of tools required to break the block

        const equipableComponent = event.player.getComponent(EntityComponentTypes.Equippable);

        if (equipableComponent) {
            const mainHandItem = equipableComponent.getEquipment(EquipmentSlot.Mainhand);

            if (tool_tiers.indexOf(mainHandItem.typeId.substring(0, mainHandItem.typeId.lastIndexOf("_") + 1)) >= tierRequiredForBreak && mainHandItem.typeId.endsWith("_pickaxe")) {
                const dropStack = new ItemStack("hightideba:blue_obsidian", 1);

                system.run(() => {
                    world.getDimension(event.block.dimension.id).spawnItem(dropStack, event.block.location);
                });
            }
        }
    }
});*/